from .requestor import Requestor
from .api_paths import *


class PrivateMessage():
    def __init__(self, _req: Requestor):
        self._req = _req
    
    def list(self, unread_only:bool=None, page:int=None, limit:int=None, instance:str=None, auth_token:str=None):
        form = {}
        optional={
            "unread_only": unread_only,
            "page": page,
            "limit": limit,
            }
        res = self._req.lemmyRequest("getPrivateMessages", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def create(self, content:str, recipient_id:int, instance:str=None, auth_token:str=None):
        form = {
            "content": content,
            "recipient_id": recipient_id,
            }
        res = self._req.lemmyRequest("createPrivateMessage", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def report(self, private_message_id:int, reason:str, instance:str=None, auth_token:str=None):
        form = {
            "private_message_id": private_message_id,
            "reason": reason,
            }
        res = self._req.lemmyRequest("createPrivateMessageReport", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def edit(self, private_message_id:int, content:str, instance:str=None, auth_token:str=None):
        form = {
            "private_message_id": private_message_id,
            "content": content,
            }
        res = self._req.lemmyRequest("editPrivateMessage", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def delete(self, private_message_id:int, instance:str=None, auth_token:str=None):
        form = {
            "private_message_id": private_message_id,
            }
        res = self._req.lemmyRequest("deletePrivateMessage", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def markAsRead(self, unread_only:bool=None, page:int=None, limit:int=None, instance:str=None, auth_token:str=None):
        form={}
        optional = {
            "unread_only": unread_only,
            "page": page,
            "limit": limit,
            }
        res = self._req.lemmyRequest("markPrivateMessageAsRead", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def listReports(self, page:int=None, limit:int=None, unresolved_only:bool=True, instance:str=None, auth_token:str=None):
        form = {}
        optional = {
            "page": page,
            "limit": limit,
            "unresolved_only": unresolved_only,
            }
        res = self._req.lemmyRequest("listPrivateMessageReports", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def resolveReport(self, report_id:int, resolved:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "report_id"  : report_id ,
            "resolved"   : resolved  ,
            }
        res = self._req.lemmyRequest("resolvePrivateMessageReport", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return


from .types.http_type import HttpType

API_VERSION = "/api/v3"
API_PATH = {
    "login" :                               { "method": HttpType.POST    , "path": "/user/login"                            },
    "getPost" :                             { "method": HttpType.GET     , "path": "/post"                                  },
    "getCommunity" :                        { "method": HttpType.GET     , "path": "/community"                             },
    "listCommunities" :                     { "method": HttpType.GET     , "path": "/community/list"                        },
    "getPosts" :                            { "method": HttpType.GET     , "path": "/post/list"                             },
    "createPost" :                          { "method": HttpType.POST    , "path": "/post"                                  },
    "editPost" :                            { "method": HttpType.PUT     , "path": "/post"                                  },
    "createComment" :                       { "method": HttpType.POST    , "path": "/comment"                               },
    "likePost" :                            { "method": HttpType.POST    , "path": "/post/like"                             },
    "likeComment" :                         { "method": HttpType.POST    , "path": "/comment/like"                          },
    "followCommunity" :                     { "method": HttpType.POST    , "path": "/community/follow"                      },
    "resolveObject":                        { "method": HttpType.GET     , "path": "/resolve_object"                        },
    "search":                               { "method": HttpType.GET     , "path": "/search"                                },
    "getSite":                              { "method": HttpType.GET     , "path": "/site"                                  },
    "leaveAdmin":                           { "method": HttpType.POST    , "path": "/user/leave_admin"                      },
    "register":                             { "method": HttpType.POST    , "path": "/user/register"                         },
    "getPersonDetails":                     { "method": HttpType.GET     , "path": "/user"                                  },
    "getPersonMentions":                    { "method": HttpType.GET     , "path": "/user/mention"                          },
    "markPersonMentionAsRead":              { "method": HttpType.POST    , "path": "/user/mention/mark_as_read"             },
    "getReplies":                           { "method": HttpType.GET     , "path": "/user/replies"                          },
    "banPerson":                            { "method": HttpType.POST    , "path": "/user/ban"                              },
    "getBannedPersons":                     { "method": HttpType.GET     , "path": "/user/banned"                           },
    "blockPerson":                          { "method": HttpType.POST    , "path": "/user/block"                            },
    "deleteAccount":                        { "method": HttpType.POST    , "path": "/user/delete_account"                   },
    "passwordReset":                        { "method": HttpType.POST    , "path": "/user/password_reset"                   },
    "passwordChangeAfterReset":             { "method": HttpType.POST    , "path": "/user/password_change"                  },
    "saveUserSettings":                     { "method": HttpType.PUT     , "path": "/user/save_user_settings"               },
    "markAllAsRead":                        { "method": HttpType.GET     , "path": "/user/mark_all_as_read"                 },
    "changePassword":                       { "method": HttpType.PUT     , "path": "/user/change_password"                  },
    "getReportCount":                       { "method": HttpType.GET     , "path": "/user/report_count"                     },
    "getUnreadCount":                       { "method": HttpType.GET     , "path": "/user/unread_count"                     },
    "verifyEmail":                          { "method": HttpType.POST    , "path": "/user/verify_email"                     },
    "addAdmin":                             { "method": HttpType.POST    , "path": "/admin/add"                             },
    "getUnreadRegistrationApplicationCount":{ "method": HttpType.GET     , "path": "/admin/registration_application/count"  },
    "listRegistrationApplications":         { "method": HttpType.GET     , "path": "/admin/registration_application/list"   },
    "createCustomEmoji":                    { "method": HttpType.GET     , "path": "/custom_emoji"                          },
    "editCustomEmoji":                      { "method": HttpType.PUT     , "path": "/custom_emoji"                          },
    "deleteCustomEmoji":                    { "method": HttpType.POST    , "path": "/custom_emoji/delete"                   },
    "addModToCommunity":                    { "method": HttpType.POST    , "path": "/community/mod"                         },
    "approveRegistrationApplication":       { "method": HttpType.PUT     , "path": "/admin/registration_application/approve"},
    "blockCommunity":                       { "method": HttpType.POST    , "path": "/community/block"                       },
    "createCommentReport":                  { "method": HttpType.POST    , "path": "/comment/report"                        },
    "createCommunity":                      { "method": HttpType.POST    , "path": "/community"                             },
    "createPostReport":                     { "method": HttpType.POST    , "path": "/post/report"                           },
    "createPrivateMessage":                 { "method": HttpType.POST    , "path": "/private_message"                       },
    "createPrivateMessageReport":           { "method": HttpType.POST    , "path": "/private_message/report"                },
    "editPrivateMessage":                   { "method": HttpType.PUT     , "path": "/private_message"                       },
    "deletePrivateMessage":                 { "method": HttpType.POST    , "path": "/private_message/delete"                },
    "markPrivateMessageAsRead":             { "method": HttpType.POST    , "path": "/private_message/mark_as_read"          },
    "deleteComment":                        { "method": HttpType.POST    , "path": "/comment/delete"                        },
    "deleteCommunity":                      { "method": HttpType.POST    , "path": "/community/delete"                      },
    "removeComment":                        { "method": HttpType.POST    , "path": "/comment/remove"                        },
    "deletePost":                           { "method": HttpType.POST    , "path": "/post/delete"                           },
    "distinguishComment":                   { "method": HttpType.POST    , "path": "/comment/distinguish"                   },
    "editComment":                          { "method": HttpType.PUT     , "path": "/comment"                               },
    "editCommunity":                        { "method": HttpType.PUT     , "path": "/community"                             },
    "banFromCommunity":                     { "method": HttpType.POST    , "path": "/community/ban_user"                    },
    "editSite":                             { "method": HttpType.PUT     , "path": "/site"                                  },
    "featurePost":                          { "method": HttpType.POST    , "path": "/post/feature"                          },
    "getFederatedInstances":                { "method": HttpType.GET     , "path": "/federated_instances"                   },
    "createSite":                           { "method": HttpType.POST    , "path": "/site"                                  },
    "getCaptcha":                           { "method": HttpType.GET     , "path": "/user/get_captcha"                      },
    "getComment":                           { "method": HttpType.GET     , "path": "/comment"                               },
    "getComments":                          { "method": HttpType.GET     , "path": "/comment/list"                          },
    "getModlog":                            { "method": HttpType.GET     , "path": "/modlog"                                },
    "getPrivateMessages":                   { "method": HttpType.GET     , "path": "/private_message/list"                  },
    "listCommentReports":                   { "method": HttpType.GET     , "path": "/comment/report/list"                   },
    "listPostReports":                      { "method": HttpType.GET     , "path": "/post/report/list"                      },
    "listPrivateMessageReports":            { "method": HttpType.GET     , "path": "/private_message/report/list"           },
    "lockPost":                             { "method": HttpType.POST    , "path": "/post/lock"                             },
    "markCommentReplyAsRead":               { "method": HttpType.POST    , "path": "/comment/mark_as_read"                  },
    "markPostAsRead":                       { "method": HttpType.POST    , "path": "/post/mark_as_read"                     },
    "purgeComment":                         { "method": HttpType.POST    , "path": "/admin/purge/comment"                   },
    "purgeCommunity":                       { "method": HttpType.POST    , "path": "/admin/purge/community"                 },
    "purgePost":                            { "method": HttpType.POST    , "path": "/admin/purge/post"                      },
    "purgePerson":                          { "method": HttpType.POST    , "path": "/admin/purge/person"                    },
    "removeCommunity":                      { "method": HttpType.POST    , "path": "/community/remove"                      },
    "resolveCommentReport":                 { "method": HttpType.PUT     , "path": "/comment/report/resolve"                },
    "resolvePostReport":                    { "method": HttpType.PUT     , "path": "/post/report/resolve"                   },
    "resolvePrivateMessageReport":          { "method": HttpType.PUT     , "path": "/private_message/report/resolve"        },
    "saveComment":                          { "method": HttpType.PUT     , "path": "/comment/save"                          },
    "savePost":                             { "method": HttpType.PUT     , "path": "/post/save"                             },
    "transferCommunity":                    { "method": HttpType.POST    , "path": "/community/transfer"                    },
    "getSiteMetadata":                      { "method": HttpType.GET     , "path": "/post/site_metadata"                    },
    "removePost":                           { "method": HttpType.POST    , "path": "/post/remove"                           },
}




from .requestor import Requestor
from .api_paths import *
from typing import List
from .types.modlog_action_type import ModlogActionType
from .types.listing_type import ListingType
from .types.registration_mode import RegistrationMode

class Site():
    def __init__(self, _req: Requestor):
        self._req = _req
    
    def getSite(self, instance:str=None, auth:bool=True, auth_token:str=None):
        form = {}
        res = self._req.lemmyRequest("getSite", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res
    
    def createSite(self,    
        name: str,
        sidebar: str=None,
        description: str=None,
        icon: str=None,
        banner: str=None,
        enable_downvotes: bool=None,
        enable_nsfw: bool=None,
        community_creation_admin_only: bool=None,
        require_email_verification: bool=None,
        application_question: str=None,
        private_instance: bool=None,
        default_theme: str=None,
        default_post_listing_type: ListingType=None,
        legal_information: str=None,
        application_email_admins: bool=None,
        hide_modlog_mod_names: bool=None,
        discussion_languages: List[int]=None,
        slur_filter_regex: str=None,
        actor_name_max_length: int=None,
        rate_limit_message: int=None,
        rate_limit_message_per_second: int=None,
        rate_limit_post: int=None,
        rate_limit_post_per_second: int=None,
        rate_limit_register: int=None,
        rate_limit_register_per_second: int=None,
        rate_limit_image: int=None,
        rate_limit_image_per_second: int=None,
        rate_limit_comment: int=None,
        rate_limit_comment_per_second: int=None,
        rate_limit_search: int=None,
        rate_limit_search_per_second: int=None,
        federation_enabled: bool=None,
        federation_debug: bool=None,
        federation_worker_count: int=None,
        captcha_enabled: bool=None,
        captcha_difficulty: str=None,
        allowed_instances: List[str]=None,
        blocked_instances: List[str]=None,
        taglines: List[str]=None,
        registration_mode: RegistrationMode=None,
        instance:str=None, 
        auth_token:str=None):
        form = {
            "name"                              : name                              ,
            }
        optional = {
            "sidebar"                           : sidebar                           ,
            "description"                       : description                       ,
            "icon"                              : icon                              ,
            "banner"                            : banner                            ,
            "enable_downvotes"                  : enable_downvotes                  ,
            "enable_nsfw"                       : enable_nsfw                       ,
            "community_creation_admin_only"     : community_creation_admin_only     ,
            "require_email_verification"        : require_email_verification        ,
            "application_question"              : application_question              ,
            "private_instance"                  : private_instance                  ,
            "default_theme"                     : default_theme                     ,
            "default_post_listing_type"         : default_post_listing_type         ,
            "legal_information"                 : legal_information                 ,
            "application_email_admins"          : application_email_admins          ,
            "hide_modlog_mod_names"             : hide_modlog_mod_names             ,
            "discussion_languages"              : discussion_languages              ,
            "slur_filter_regex"                 : slur_filter_regex                 ,
            "actor_name_max_length"             : actor_name_max_length             ,
            "rate_limit_message"                : rate_limit_message                ,
            "rate_limit_message_per_second"     : rate_limit_message_per_second     ,
            "rate_limit_post"                   : rate_limit_post                   ,
            "rate_limit_post_per_second"        : rate_limit_post_per_second        ,
            "rate_limit_register"               : rate_limit_register               ,
            "rate_limit_register_per_second"    : rate_limit_register_per_second    ,
            "rate_limit_image"                  : rate_limit_image                  ,
            "rate_limit_image_per_second"       : rate_limit_image_per_second       ,
            "rate_limit_comment"                : rate_limit_comment                ,
            "rate_limit_comment_per_second"     : rate_limit_comment_per_second     ,
            "rate_limit_search"                 : rate_limit_search                 ,
            "rate_limit_search_per_second"      : rate_limit_search_per_second      ,
            "federation_enabled"                : federation_enabled                ,
            "federation_debug"                  : federation_debug                  ,
            "federation_worker_count"           : federation_worker_count           ,
            "captcha_enabled"                   : captcha_enabled                   ,
            "captcha_difficulty"                : captcha_difficulty                ,
            "allowed_instances"                 : allowed_instances                 ,
            "blocked_instances"                 : blocked_instances                 ,
            "taglines"                          : taglines                          ,
            "registration_mode"                 : registration_mode                 ,
            }
        res = self._req.lemmyRequest("createSite", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    
    def editSite(self,         
        name: str,
        sidebar: str=None,
        description: str=None,
        icon: str=None,
        banner: str=None,
        enable_downvotes: bool=None,
        enable_nsfw: bool=None,
        community_creation_admin_only: bool=None,
        require_email_verification: bool=None,
        application_question: str=None,
        private_instance: bool=None,
        default_theme: str=None,
        default_post_listing_type: ListingType=None,
        legal_information: str=None,
        application_email_admins: bool=None,
        hide_modlog_mod_names: bool=None,
        discussion_languages: List[int]=None,
        slur_filter_regex: str=None,
        actor_name_max_length: int=None,
        rate_limit_message: int=None,
        rate_limit_message_per_second: int=None,
        rate_limit_post: int=None,
        rate_limit_post_per_second: int=None,
        rate_limit_register: int=None,
        rate_limit_register_per_second: int=None,
        rate_limit_image: int=None,
        rate_limit_image_per_second: int=None,
        rate_limit_comment: int=None,
        rate_limit_comment_per_second: int=None,
        rate_limit_search: int=None,
        rate_limit_search_per_second: int=None,
        federation_enabled: bool=None,
        federation_debug: bool=None,
        federation_worker_count: int=None,
        captcha_enabled: bool=None,
        captcha_difficulty: str=None,
        allowed_instances: List[str]=None,
        blocked_instances: List[str]=None,
        taglines: List[str]=None,
        registration_mode: RegistrationMode=None,
        reports_email_admins:bool=None,
        instance:str=None, 
        auth_token:str=None):
        form = {
            }
        optional = {
            "name"                              : name                              ,
            "sidebar"                           : sidebar                           ,
            "description"                       : description                       ,
            "icon"                              : icon                              ,
            "banner"                            : banner                            ,
            "enable_downvotes"                  : enable_downvotes                  ,
            "enable_nsfw"                       : enable_nsfw                       ,
            "community_creation_admin_only"     : community_creation_admin_only     ,
            "require_email_verification"        : require_email_verification        ,
            "application_question"              : application_question              ,
            "private_instance"                  : private_instance                  ,
            "default_theme"                     : default_theme                     ,
            "default_post_listing_type"         : default_post_listing_type         ,
            "legal_information"                 : legal_information                 ,
            "application_email_admins"          : application_email_admins          ,
            "hide_modlog_mod_names"             : hide_modlog_mod_names             ,
            "discussion_languages"              : discussion_languages              ,
            "slur_filter_regex"                 : slur_filter_regex                 ,
            "actor_name_max_length"             : actor_name_max_length             ,
            "rate_limit_message"                : rate_limit_message                ,
            "rate_limit_message_per_second"     : rate_limit_message_per_second     ,
            "rate_limit_post"                   : rate_limit_post                   ,
            "rate_limit_post_per_second"        : rate_limit_post_per_second        ,
            "rate_limit_register"               : rate_limit_register               ,
            "rate_limit_register_per_second"    : rate_limit_register_per_second    ,
            "rate_limit_image"                  : rate_limit_image                  ,
            "rate_limit_image_per_second"       : rate_limit_image_per_second       ,
            "rate_limit_comment"                : rate_limit_comment                ,
            "rate_limit_comment_per_second"     : rate_limit_comment_per_second     ,
            "rate_limit_search"                 : rate_limit_search                 ,
            "rate_limit_search_per_second"      : rate_limit_search_per_second      ,
            "federation_enabled"                : federation_enabled                ,
            "federation_debug"                  : federation_debug                  ,
            "federation_worker_count"           : federation_worker_count           ,
            "captcha_enabled"                   : captcha_enabled                   ,
            "captcha_difficulty"                : captcha_difficulty                ,
            "allowed_instances"                 : allowed_instances                 ,
            "blocked_instances"                 : blocked_instances                 ,
            "taglines"                          : taglines                          ,
            "registration_mode"                 : registration_mode                 ,
            "reports_email_admins"              : reports_email_admins              ,
            }
        res = self._req.lemmyRequest("editSite", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def getFederatedInstances(self, instance:str=None, auth:bool=True, auth_token:str=None):
        res = self._req.lemmyRequest("getFederatedInstances", instance=instance, auth=auth, auth_token=auth_token)
        return res
        
    def createCustomEmoji(self, category:str, shortcode:str, image_url:str, alt_text:str, keywords:List[str], instance:str=None, auth_token:str=None):
        form={
            "category": category,
            "shortcode": shortcode,
            "image_url": image_url,
            "alt_text": alt_text,
            "keywords": keywords
            }
        res = self._req.lemmyRequest("createCustomEmoji", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res
        
    def editCustomEmoji(self, emoji_id:int, category:str=None, image_url:str=None, alt_text:str=None, keywords:List[str]=None, instance:str=None, auth_token:str=None):
        form={
            "id": emoji_id,
            "category": category,
            "image_url": image_url,
            "alt_text": alt_text,
            "keywords": keywords
            }
        res = self._req.lemmyRequest("editCustomEmoji", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res
        
    def deleteCustomEmoji(self, emoji_id:int, instance:str=None, auth_token:str=None):
        form={
            "id": emoji_id
            }
        res = self._req.lemmyRequest("deleteCustomEmoji", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res

    def addAdmin(self, person_id:int, added:bool, instance:str=None, auth_token:str=None):
        form={
            "person_id": person_id,
            "added": added
            }
        res = self._req.lemmyRequest("addAdmin", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def getUnreadRegistrationApplicationCount(self, instance:str=None, auth_token=None):
        form={}
        res = self._req.lemmyRequest("getUnreadRegistrationApplicationCount", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def listRegistrationApplications(self, unread_only:bool=None, page:int=None, limit:str=None, instance:str=None, auth_token=None):
        form={}
        optional={
            "page": page,
            "limit": limit,
            "unread_only": unread_only
            }
        res = self._req.lemmyRequest("listRegistrationApplications", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def approveRegistrationApplication(self, application_id:int, approve:bool, deny_reason:str=None, instance:str=None, auth_token:str=None):
        form={
            "application_id": application_id,
            "approve": approve,
            }
        optional={
            "deny_reason": deny_reason
            }
        res = self._req.lemmyRequest("approveRegistrationApplication", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def getModlog(self, mod_person_id:int=None, community_id:int=None, page:int=None, limit:int=None, type_:ModlogActionType=None, other_person_id:int=None, instance:str=None, auth_token:str=None):
        form={}
        optional={
            "mod_person_id": mod_person_id,
            "community_id": community_id,
            "page": page,
            "limit": limit,
            "type_": type_,
            "other_person_id": other_person_id,
            }
        res = self._req.lemmyRequest("getModlog", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def purgeComment(self, comment_id:int, reason:str=None, instance:str=None, auth_token:str=None):
        form={
            "comment_id": comment_id
            }
        optional={
            "reason": reason,
            }
        res = self._req.lemmyRequest("purgeComment", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def purgeCommunity(self, community_id:int, reason:str=None, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id
            }
        optional={
            "reason": reason,
            }
        res = self._req.lemmyRequest("purgeCommunity", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def purgePost(self, post_id:int, reason:str=None, instance:str=None, auth_token:str=None):
        form={
            "post_id": post_id
            }
        optional={
            "reason": reason,
            }
        res = self._req.lemmyRequest("purgePost", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def purgePerson(self, person_id:int, reason:str=None, instance:str=None, auth_token:str=None):
        form={
            "person_id": person_id
            }
        optional={
            "reason": reason,
            }
        res = self._req.lemmyRequest("purgePerson", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res


from .requestor import Requestor
from .comment import Comment
from .community import Community
from .post import Post
from .user import User
from .site import Site
from .private_message import PrivateMessage

class Lemmy:
    Post: Post
    Community: Community
    Comment: Comment
    User: User
    Site: Site
    PrivateMessage: PrivateMessage
    
    def __enter__(self):
        """Handle the context manager open."""
        return self
    
    def __exit__(self, *_args):
        """Handle the context manager close."""
    
    def __init__(self, instance, username:str = None, password:str = None):
        self.username = username
        
        # Login, get token, and set as header for future
        self._req = Requestor(instance=instance, username=username, password=password, headers={})
        
        self.Post = Post(self._req)
        self.Community = Community(self._req)
        self.Comment = Comment(self._req)
        self.User = User(self._req)
        self.Site = Site(self._req)
        self.PrivateMessage = PrivateMessage(self._req)
        # print(self._req.headers.get("Authorization"))
        
    def resolveObject(self, obj, instance=None, auth_token=None):
        form={
            "q": obj
        }
        res = self._req.lemmyRequest("resolveObject", instance=instance, form=form, auth_token=auth_token)
        return res
        
    def search(self, term, instance=None, auth_token=None):
        form={
            "q": term
        }
        res = self._req.request("search", instance=instance, auth_token=auth_token, form=form)
        return res



from .comment_sort_type import CommentSortType
from .http_type import HttpType
from .listing_type import ListingType
from .modlog_action_type import ModlogActionType
from .post_feature_type import PostFeatureType
from .search_type import SearchType
from .sort_type import SortType
from .subscribed_type import SubscribedType


from enum import Enum
class SortType(Enum):
    Active=         "Active"
    Hot=            "Hot"
    New=            "New"
    Old=            "Old"
    TopDay=         "TopDay"
    TopWeek=        "TopWeek"
    TopMonth=       "TopMonth"
    TopYear=        "TopYear"
    TopAll=         "TopAll"
    MostComments=   "MostComments"
    NewComments=    "NewComments"
    TopHour=        "TopHour"
    TopSixHour=     "TopSixHour"
    TopTwelveHour=  "TopTwelveHour"



from .requestor import Requestor
from .api_paths import *
from typing import List

class Community():
    def __init__(self, _req: Requestor):
        self._req = _req
        
    def get(self, name:str, instance:str=None, auth:bool=True, auth_token:str=None):
        form = {
            "name": name
        }
        res = self._req.lemmyRequest("getCommunity", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res["community_view"]
        
    def list(self, instance:str=None, auth:bool=True, auth_token:str=None): 
        form={}
        res = self._req.lemmyRequest("listCommunities", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res["communities"]
        
    def follow(self, community_id:int, follow:bool=True, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            "follow": follow
        }
        res = self._req.lemmyRequest("followCommunity", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res["community_view"]
    def addMod(self, community_id:int, person_id:int, added:bool=True, instance:str=None, auth_token:str=None): 
        form={
            "community_id": community_id,
            "person_id": person_id,
            "added": added,
        }
        res = self._req.lemmyRequest("addModToCommunity", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def banPerson(self, community_id:int, person_id:int, ban:bool, remove_data:bool=None, reason:str=None, expires:int=None, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            "person_id": person_id,
            "ban": ban,
        }
        optional={
            "remove_data": remove_data,
            "reason": reason,
            "expires": expires,
        }
        res = self._req.lemmyRequest("banFromCommunity", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def block(self, community_id:int, block:bool=True, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            "block": block,
            }
        res = self._req.lemmyRequest("blockCommunity", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res["community_view"]
        
    def create(self, name:str, title:str, description:str=None, icon:str=None, banner:str=None, nsfw:bool=None, posting_restricted_to_mods:bool=None, discussion_languages:List[str]=None, instance:str=None, auth_token:str=None):
        form={
            "name": name,
            "title": title,
            }
        optional={
            "description": description,
            "icon": icon,
            "banner": banner,
            "nsfw": nsfw,
            "posting_restricted_to_mods": posting_restricted_to_mods,
            "discussion_languages": discussion_languages,
            }
        res = self._req.lemmyRequest("createCommunity", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res["community_view"]
        
    def delete(self, community_id:int, deleted:bool=True, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            "deleted": deleted,
            }
        res = self._req.lemmyRequest("deleteCommunity", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res["community_view"]
    
    def edit(self, community_id:int, title:str=None, description:str=None, icon:str=None, banner:str=None, nsfw:bool=None, posting_restricted_to_mods:bool=None, discussion_languages:List[str]=None, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            }
        optional={
            "title": title,
            "description": description,
            "icon": icon,
            "banner": banner,
            "nsfw": nsfw,
            "posting_restricted_to_mods": posting_restricted_to_mods,
            "discussion_languages": discussion_languages,
            }
        res = self._req.lemmyRequest("createCommunity", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res["community_view"]
        
    def remove(self, community_id:int, removed:bool=True, reason:str=None, expires:int=None, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            "removed":removed,
            }
        optional={
            "reason": reason,
            "expires": expires,
            }
        res = self._req.lemmyRequest("removeCommunity", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res["community_view"]
    
    def transfer(self, community_id:int, person_id:int, instance:str=None, auth_token:str=None):
        form={
            "community_id": community_id,
            "person_id":person_id,
            }
        res = self._req.lemmyRequest("transferCommunity", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res["community_view"]


from .requestor import Requestor
from .api_paths import *
from .types.post_feature_type import PostFeatureType

class Post():
    def __init__(self, _req: Requestor):
        self._req = _req
        
    def list(self, community_id:int, sort:str=None, instance:str=None, auth:bool=True, auth_token:str=None):
        form = { "sort": sort or "New", "community_id": community_id }
        res = self._req.lemmyRequest("getPosts", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res["posts"]
        
    def get(self, community_id, post_id, instance:str=None, auth=True, auth_token:str=None):
        form = {
            "id": post_id,
        }
        res = self._req.lemmyRequest("getPost", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res["post_view"]
        
    def create(self, community_id:int, title:str=None, body:str=None, remote_url:str=None, nsfw=False, language_id=None, instance:str=None, auth_token:str=None):
        #there could be a bug here if instance != logged in instance when trying to create a post
        form = {
            "community_id": community_id,
            "name": title,
            "body": body,
            "language_id": language_id,
            "nsfw": nsfw
            }
        optional={
            "url": remote_url
        }
        res = self._req.lemmyRequest("createPost", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res["post_view"]
        
    def edit(self, post_id:int, title:str=None, body:str=None, remote_url:str=None, nsfw:bool=None, language_id=None, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
        }
        optional={
            "url": remote_url,
            "nsfw": nsfw,
            "language_id": language_id,
            "name": name,
            "body": body
        }
        res = self._req.lemmyRequest("editPost", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res["post_view"]
        
    def like(self, post_id:int, score:int=1, language_id=None, instance:str=None, auth_token:str=None):
        if(score > 1):
            score = 1
        if(score < -1):
            score = -1
        form = {
            "post_id": post_id,
            "score": score
        }
        res = self._req.lemmyRequest("likePost", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def report(self, post_id:int, reason:str, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
            "reason": reason
            }
        res = self._req.lemmyRequest("createPostReport", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def delete(self, post_id:int, deleted:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
            "deleted": deleted
            }
        res = self._req.lemmyRequest("deletePost", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def feature(self, post_id:int, feature_type: PostFeatureType, featured:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
            "feature_type": feature_type.value,
            "featured": featured,
            }
        res = self._req.lemmyRequest("featurePost", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def listReports(self, page:int=None, limit:int=None, unresolved_only:bool=True, community_id:int=None, instance:str=None, auth_token:str=None):
        form = {}
        optional = {
            "page": page,
            "limit": limit,
            "unresolved_only": unresolved_only,
            "community_id": community_id,
            }
        res = self._req.lemmyRequest("listPostReports", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    
    def lock(self, post_id:int, locked:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
            "locked": locked,
        }
        res = self._req.lemmyRequest("lockPost", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def markAsRead(self, post_id:int, read:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
            "read": read,
        }
        res = self._req.lemmyRequest("markPostAsRead", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def resolveReport(self, report_id:int, resolved:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "report_id"  : report_id ,
            "resolved"   : resolved  ,
            }
        res = self._req.lemmyRequest("resolvePostReport", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return
        
    def save(self, post_id:int, save:bool=True, instance:str=None, auth_token:str=None):
        form = {
            "post_id"  : post_id ,
            "save"   : save,
            }
        res = self._req.lemmyRequest("savePost", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return
        
    def getSiteMetadata(self, remote_url:str, instance:str=None, auth:bool=True, auth_token:str=None):
        form={
            "url": remote_url,
        }
        res = self._req.lemmyRequest("getSiteMetadata", instance=instance, form=form, auth=auth, auth_token=auth_token)
        return res
        
    def remove(self, post_id:int, mod_person_id:int, when_:str, removed:bool=True, reason:str=None, instance:str=None, auth_token:str=None):
        form = {
            "post_id": post_id,
            "mod_person_id": mod_person_id,
            "when_": when_,
            "removed": removed,
            }
        optional={
            "reason":reason,
            }
        res = self._req.lemmyRequest("removePost", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res["post_view"]


from .requestor import Requestor
from .api_paths import *
from typing import List

class User():
    def __init__(self, _req: Requestor):
        self._req = _req
    def leaveAdmin(self, instance:str=None, auth_token:str=None):
        form = {}
        res = self._req.lemmyRequest("leaveAdmin", instance=instance, form=form, auth=True, auth_token=auth_token)
    def register(self, username:str, password:str, password_verify:str, show_nsfw:bool, email:str=None, captcha_uuid:str=None, captcha_answer:str=None, honeypot:str=None, answer:str=None, instance:str=None):
        form={
            "username": username,
            "password": password,
            "password_verify": password_verify,
            "show_nsfw": show_nsfw
            }
        optional = {       
            "email"          :email          ,
            "captcha_uuid"   :captcha_uuid   ,
            "captcha_uuid"   :captcha_uuid   ,
            "limit"          :limit          ,
            "honeypot"       :honeypot       ,
            "answer"         :answer         ,
            }
        res = self._req.lemmyRequest("register", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def getDetails(self, person_id:str=None, username:str=None, sort:str=None, page:int=0, limit:int=10, community_id:int=None, saved_only:bool=None, instance:str=None, auth:bool=True, auth_token:str=None):
        form={}
        optional={
            "person_id"      :person_id      ,
            "username"       :username       ,
            "sort"           :sort           ,
            "page"           :page           ,
            "limit"          :limit          ,
            "community_id"   :community_id   ,
            "saved_only"     :saved_only     ,
            }
        res = self._req.lemmyRequest("getPersonDetails", instance=instance, form=form, optional=optional, auth=auth, auth_token=auth_token)
        return res
    
    def markMentionAsRead(self, person_mention_id:int, read:bool=True, instance:str=None, auth_token:str=None):
        form={
            "person_mention_id": person_mention_id,
            "read": read
            }
        res = self._req.lemmyRequest("markPersonMentionAsRead", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def getMentions(self, sort:str=None, page:int=0, limit:int=10, unread_only:bool=True, instance:str=None, auth_token:str=None):
        form={}
        optional = {
            "sort"           :sort           ,
            "page"           :page           ,
            "limit"          :limit          ,
            "unread_only"    :unread_only    ,
            }
        res = self._req.lemmyRequest("getPersonMentions", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    
    def getReplies(self, sort:str=None, page:int=0, limit:int=10, unread_only:bool=True, auth_token:str=None):
        form={}
        optional = {
            "sort"           :sort           ,
            "page"           :page           ,
            "limit"          :limit          ,
            "unread_only"    :unread_only    ,
            }
        res = self._req.lemmyRequest("getReplies", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    
    def ban(self, person_id:int, ban:bool, remove_data:bool=None, reason:str=None, expires:int=None, instance:str=None, auth_token:str=None):
        form={
            "person_id"     :person_id,
            "ban"           :ban
            }
        optional = {
            "remove_data"    :remove_data   ,
            "reason"         :reason        ,
            "expires"        :expires
            }
        res = self._req.lemmyRequest("banPerson", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    
    def getBanned(self, instance:str=None, auth_token:str=None):
        form={}
        res = self._req.lemmyRequest(HttpType.GET, "getBannedPersons", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
     
    def block(self, person_id:int, block:bool, instance:str=None, auth_token:str=None):
        form={
            "person_id": person_id,
            "block": block
            }
        res = self._req.lemmyRequest("blockPerson", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def getCaptcha(self, instance:str=None, auth_token:str=None):
        form={}
        res = self._req.lemmyRequest("getCaptcha", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def deleteAccount(self, password, instance:str=None, auth_token:str=None):
        form={
            "password": password
            }
        res = self._req.lemmyRequest("deleteAccount", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
    
    def passwordReset(self, email, instance:str=None, auth_token:str=None):
        form={
            "email": email
            }
        res = self._req.lemmyRequest("passwordReset", instance=instance, form=form, auth=False, auth_token=auth_token)
        return res
        
    def passwordChangeAfterReset(self, token:str, password:str, password_verify:str, instance:str=None, auth_token:str=None):
        form={
            "token": token,
            "password": password,
            "password_verify": password_verify
            }
        res = self._req.lemmyRequest("passwordChangeAfterReset", instance=instance, form=form, auth=False, auth_token=auth_token)
        return res
        
    def markAllAsRead(self, community_view:str=None, site:str=None, moderators:List[str]=None, discussion_languages:List[str]=None, instance:str=None, auth_token:str=None):
        form={}
        optional={
            "community_view": community_view,
            "site": site,
            "moderators": moderators,
            "discussion_languages": discussion_languages,
            }
        res = self._req.lemmyRequest("markAllAsRead", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def saveUserSettings(self,
        show_nsfw: bool=None,
        show_scores: bool=None,
        theme: str=None,
        default_sort_type: str=None,
        default_listing_type: str=None,
        interface_language: str=None,
        avatar: str=None,
        banner: str=None,
        display_name: str=None,
        email: str=None,
        bio: str=None,
        matrix_user_id: str=None,
        show_avatars: bool=None,
        send_notifications_to_email: bool=None,
        bot_account: bool=None,
        show_bot_accounts: bool=None,
        show_read_posts: bool=None, 
        show_new_post_notifs: bool=None, 
        discussion_languages: List[str]=None, 
        generate_totp_2fa: bool=None, 
        instance:str=None,
        auth_token:str=None):
        form = {}
        optional = {
            "show_nsfw"                     : show_nsfw                  ,
            "show_scores"                   : show_scores                ,
            "theme"                         : theme                      ,
            "default_sort_type"             : default_sort_type          ,
            "default_listing_type"          : default_listing_type       ,
            "interface_language"            : interface_language         ,
            "avatar"                        : avatar                     ,
            "banner"                        : banner                     ,
            "display_name"                  : display_name               ,
            "email"                         : email                      ,
            "bio"                           : bio                        ,
            "matrix_user_id"                : matrix_user_id             ,
            "show_avatars"                  : show_avatars               ,
            "send_notifications_to_email"   : send_notifications_to_email,
            "bot_account"                   : bot_account                ,
            "show_bot_accounts"             : show_bot_accounts          ,
            "show_read_posts"               : show_read_posts            ,
            "show_new_post_notifs"          : show_new_post_notifs       ,
            "discussion_languages"          : discussion_languages       ,
            "generate_totp_2fa"             : generate_totp_2fa          
            }
        res = self._req.lemmyRequest("saveUserSettings", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
    def changePassword(self, 
        new_password: str,
        new_password_verify: str,
        old_password: str,
        instance:str=None,
        auth_token:str=None):
        form = {
            "new_password": new_password,
            "new_password_verify" : new_password_verify,
            "old_password": old_password
        }
        res = self._req.lemmyRequest("changePassword", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def getReportCount(self,community_id:int=None,  instance:str=None, auth_token:str=None):
        form={}
        optional={
            "community_id": community_id
            }
        res = self._req.lemmyRequest("getReportCount", instance=instance, form=form, optional=optional, auth=True, auth_token=auth_token)
        return res
        
    def getUnreadCount(self, instance:str=None, auth_token:str=None):
        form={}
        res = self._req.lemmyRequest("getUnreadCount", instance=instance, form=form, auth=True, auth_token=auth_token)
        return res
        
    def verifyEmail(self, instance:str=None, token:str=None):
        form={ "token": token }
        res = self._req.lemmyRequest("verifyEmail", instance=instance, form=form, auth=False, auth_token=auth_token)
        return res